# -*- coding: utf-8 -*-
"""
This example demonstrates the use of pyqtgraph's parametertree system. This provides
a simple way to generate user interfaces that control sets of parameters. The example
demonstrates a variety of different parameter types (int, float, list, etc.)
as well as some customized parameter types

"""


import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui
import pyqtgraph.parametertree.parameterTypes as pTypes
from pyqtgraph.parametertree import Parameter, ParameterTree, ParameterItem, registerParameterType



class BasicAcquisitonParameters(pTypes.GroupParameter):
    def __init__(self, **opts):
        super().__init__(**opts)
        
        self.addChild({'name': 'Camera name', 'type': 'str', 'value': ' ', 'readonly': True})
        self.addChild({'name': 'Acquisition mode', 'type': 'list', 'values': ['acq1', 'acq2']})
        self.addChild({'name': 'Pixel Binning', 'type': 'list', 'values': ['1x1', '2x2', '4x4', '8x8']})
        self.addChild({'name': 'Image Area', 'type': 'list', 'values': ['2048x2048', '1024x1024', '512x512', '256x256'], 'readonly': True})
        self.addChild({'name': 'Exposure Time', 'type': 'float', 'value': .1, 'suffix': 's', 'siPrefix': True, 'step': 0.05})
        self.addChild({'name': 'Frame Rate', 'type': 'float', 'value': 0, 'readonly': True})
        
                

        #exposure time and frame rate connected because they are the inverse of each other
        #later, they should be set so that the framerate is the maximum possible value given an exposure time
        # self.exposureTimeWidget = self.param('Exposure Time')
        # self.frameRateWidget = self.param('Frame Rate')
        # self.exposureTimeWidget.sigValueChanged.connect(self.expTimeChanged)
        # self.frameRateWidget.sigValueChanged.connect(self.frameRateChanged)
        
        #set image area according to binning
        self.param('Pixel Binning').sigValueChanged.connect(self.binningChanged)
            
    # def expTimeChanged(self):
    #     self.frameRateWidget.setValue(1.0 / self.exposureTimeWidget.value(), blockSignal=self.frameRateChanged)

    # def frameRateChanged(self):
    #     self.exposureTimeWidget.setValue(1.0 / self.frameRateWidget.value(), blockSignal=self.expTimeChanged)

    def binningChanged(self):
        if self.param('Pixel Binning').value() == '1x1':
            self.param('Image Area').setValue('2048x2048')
        if self.param('Pixel Binning').value() == '2x2':
            self.param('Image Area').setValue('1024x1024')
        if self.param('Pixel Binning').value() == '4x4':
            self.param('Image Area').setValue('512x512')
        if self.param('Pixel Binning').value() == '8x8':
            self.param('Image Area').setValue('256x256')


class BasicActionParameters(pTypes.GroupParameter):
    def __init__(self, **opts):
        super().__init__(**opts)
        self.addChild({'name': 'Save State', 'type': 'action'})
        self.addChild({'name': 'Restore State', 'type': 'action'})


class CameraParameterTree(ParameterTree):
    def __init__(self):
        super().__init__()
        self.params = [
                BasicAcquisitonParameters(name = 'Basic acq parameters'),
                BasicActionParameters(name = 'Save/Restore functionality')
                ]
        self.p = Parameter.create(name='params', type='group', children=self.params)


        self.p.sigTreeStateChanged.connect(self.change)
        self.p.param('Save/Restore functionality', 'Save State').sigActivated.connect(self.save)
        self.p.param('Save/Restore functionality', 'Restore State').sigActivated.connect(self.restore)
        
        self.t = ParameterTree()
        self.t.setParameters(self.p, showTop=False)
        self.t.setWindowTitle('Parameter Tree')
        
        self.save()
        
    def change(self, param, changes):
        print("tree changes:")
        for param, change, data in changes:
            path = self.p.childPath(param)
            if path is not None:
                childName = '.'.join(path)
            else:
                childName = param.name()
            print('  parameter: %s'% childName)
            print('  change:    %s'% change)
            print('  data:      %s'% str(data))
            print('  ----------')

    def save(self):
        self.state = self.p.saveState()
        
    def restore(self):
        self.p.restoreState(self.state)


## Start Qt event loop unless running in interactive mode or using pyside.
if __name__ == '__main__':
    app = QtGui.QApplication([])

    import sys
    param_tree = CameraParameterTree()
    win = QtGui.QWidget()
    layout = QtGui.QGridLayout()
    win.setLayout(layout)
    layout.addWidget(QtGui.QLabel("Explanation of parameter tree."), 0,  0, 1, 2)
    layout.addWidget(param_tree.t)
    win.show()
    win.resize(400,800)
    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
        QtGui.QApplication.instance().exec_()
